@@ -63,11 +63,11 @@ __all__ = [
 ]
 
 import contextlib
-import logging
 import os
 import warnings
 from pathlib import Path
 
+from egse.log import logger
 from egse.system import all_logging_disabled
 from egse.system import get_caller_info
 from egse.system import ignore_m_warning
@@ -75,7 +75,6 @@ from egse.system import ignore_m_warning
 from rich.console import Console
 
 console = Console(width=100)
-logger = logging.getLogger("egse.env")
 
 # Every project shall have a PROJECT and a SITE_ID environment variable set. This variable will be used to
 # create the other environment variables that are specific to the project.
@@ -484,7 +483,7 @@ def get_local_settings_path() -> str or None:
 
 def has_conf_repo_location() -> bool:
     location = _env.get("CONF_REPO_LOCATION")
-    return True if location else False
+    return True if location in (None, NoValue) else False
 
 
 def get_conf_repo_location_env_name() -> str:
