@@ -1,8 +1,5 @@
 """
 This module implements a standard Observer <-> Observable pattern.
-
-!!! Bug
-    The methods in the Observable are not PEP8 compliant and need to be deprecated and changed.
 """
 
 import abc
@@ -26,26 +23,26 @@ class Observable:
     def __init__(self):
         self.observers = []
 
-    def addObserver(self, observer):
+    def add_observer(self, observer):
         if observer not in self.observers:
             self.observers.append(observer)
 
-    def deleteObserver(self, observer):
+    def delete_observer(self, observer):
         self.observers.remove(observer)
 
-    def clearObservers(self):
+    def clear_observers(self):
         self.observers = []
 
-    def countObservers(self):
+    def count_observers(self):
         return len(self.observers)
 
-    def notifyObservers(self, changedObject):
+    def notify_observers(self, changed_object):
         # FIXME: put a try..except here to log any problem that occurred in the observer's update()
         #        method
         for observer in self.observers:
-            observer.update(changedObject)
+            observer.update(changed_object)
 
-    def actionObservers(self, actions):
+    def action_observers(self, actions):
         # FIXME: put a try..except here to log any problem that occurred in the observer's do()
         #        method
         for observer in self.observers:
