@@ -29,10 +29,10 @@ import operator
 import os
 import platform  # For getting the operating system name
 import re
-import signal
 import socket
 import subprocess  # For executing a shell command
 import sys
+import threading
 import time
 import warnings
 from collections import namedtuple
@@ -58,11 +58,12 @@ from rich.text import Text
 from rich.tree import Tree
 from typer.core import TyperCommand
 
+import signal
+from egse.log import logger
+
 EPOCH_1958_1970 = 378691200
 TIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f%z"
 
-logger = logging.getLogger(__name__)
-
 
 # ACKNOWLEDGEMENT: The class is based on the textual.timer.Timer class from the Textual project.
 class Periodic:
@@ -87,14 +88,14 @@ class Periodic:
     """The number of Periodic instances that are created."""
 
     def __init__(
-            self,
-            interval: float,
-            *,
-            name: str | None = None,
-            callback: Callable = None,
-            repeat: int | None = None,
-            skip: bool = True,
-            pause: bool = False,
+        self,
+        interval: float,
+        *,
+        name: str | None = None,
+        callback: Callable = None,
+        repeat: int | None = None,
+        skip: bool = True,
+        pause: bool = False,
     ) -> None:
         self._interval = interval
         self.name = f"Periodic#{self._periodic_count}" if name is None else name
@@ -232,7 +233,7 @@ def round_up(n: float | int, decimals: int = 0):
         For negative numbers, "rounding up" means rounding toward zero,
         so -3.14159 rounded up to 3 decimals is -3.141.
     """
-    multiplier = 10 ** decimals
+    multiplier = 10**decimals
     return math.ceil(n * multiplier) / multiplier
 
 
@@ -265,6 +266,7 @@ class TyperAsyncCommand(TyperCommand):
             ...
 
     """
+
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
@@ -373,7 +375,7 @@ def now(utc: bool = True):
 
 
 def format_datetime(
-        dt: Union[str, datetime.datetime] = None, fmt: str = None, width: int = 6, precision: int = 3
+    dt: Union[str, datetime.datetime] = None, fmt: str = None, width: int = 6, precision: int = 3
 ) -> str:
     """Format a datetime as YYYY-mm-ddTHH:MM:SS.μs+0000.
 
@@ -1074,8 +1076,7 @@ def get_os_version() -> str:
 
 
 def wait_until(
-        condition: Callable, *args: list, interval: float = 0.1, timeout: float = 1.0, verbose: bool = False,
-        **kwargs: dict
+    condition: Callable, *args: list, interval: float = 0.1, timeout: float = 1.0, verbose: bool = False, **kwargs: dict
 ) -> bool:
     """
     Sleep until the given condition is fulfilled. The arguments are passed into the condition
@@ -1135,8 +1136,7 @@ def wait_until(
 
 
 def waiting_for(
-        condition: Callable, *args: list, interval: float = 0.1, timeout: float = 1.0, verbose: bool = False,
-        **kwargs: dict
+    condition: Callable, *args: list, interval: float = 0.1, timeout: float = 1.0, verbose: bool = False, **kwargs: dict
 ) -> float:
     """
     Sleep until the given condition is fulfilled. The arguments are passed into the condition
@@ -1229,7 +1229,15 @@ def has_internet(host: str = "8.8.8.8", port: int = 53, timeout: float = 3.0):
             s.close()
 
 
-def do_every(period: float, func: callable, *args: tuple[int, ...], count: int = None) -> None:
+def do_every(
+    period: float,
+    func: Callable,
+    *args: tuple[int, ...],
+    count: int = None,
+    setup_func: Callable = None,
+    teardown_func: Callable = None,
+    stop_event: threading.Event = None,
+) -> None:
     """
     This method executes a function periodically, taking into account
     that the function that is executed will take time also and using a
@@ -1245,12 +1253,42 @@ def do_every(period: float, func: callable, *args: tuple[int, ...], count: int =
     timer_thread.start()
     ```
 
+    The `setup_func` and `teardown` functions will be called before and after
+    the loop that repeats the `func` function. This can be used e.g. for setting
+    up and closing sockets.
+
+    Apart from the `count`, the loop can also be stopped by passing a threading
+    event and setting the `stop_event` when you want to terminate the thread.
+
+    ```
+    self._stop_event = threading.Event()
+
+    timer_thread = threading.Thread(
+        target=do_every,
+        args=(interval, send_heartbeat),
+        kwargs={
+            'stop_event': self._stop_event,
+            'setup_func': self._connect_hb_socket,
+            'teardown_func': self._disconnect_hb_socket
+        }
+    )
+    timer_thread.daemon = True
+    timer_thread.start()
+
+    ...
+
+    self._stop_event.set()
+    ```
+
     Args:
         period: a time interval between successive executions [seconds]
         func: the function to be executed
         *args: optional arguments to be passed to the function
         count: if you do not need an endless loop, provide the number of
             iterations, if count=0 the function will not be executed.
+        setup_func: a function that will be called before going into the loop
+        teardown_func: a function that will be called when the loop ended
+        stop_event: use a threading event to stop the loop
     """
 
     # Code from SO:https://stackoverflow.com/a/28034554/4609203
@@ -1267,13 +1305,25 @@ def do_every(period: float, func: callable, *args: tuple[int, ...], count: int =
     g = g_tick()
     iteration = 0
 
-    while True:
+    if stop_event is None:
+        stop_event = threading.Event()
+
+    if setup_func:
+        setup_func()
+
+    while not stop_event.is_set():
         if count is not None and iteration >= count:
             break
-        time.sleep(next(g))
+        # Wait for the timeout or until the stop_event is set
+        # The wait functions returns True only when the event is set and returns False on a timeout
+        if stop_event.wait(timeout=next(g)):
+            break
         func(*args)
         iteration += 1
 
+    if teardown_func:
+        teardown_func()
+
 
 @contextlib.contextmanager
 def chdir(dirname=None):
@@ -1885,7 +1935,7 @@ SIGNAL_NAME = {
     30: "SIGUSR1",
     31: "SIGUSR2",
 }
-"""The signals that can be catch with the SignalCatcher."""
+"""The signals that can be caught with the SignalCatcher."""
 
 
 class SignalCatcher:
@@ -2124,7 +2174,6 @@ def capture_rich_output(obj: Any, width: int = 120) -> str:
 
 
 def log_rich_output(logger_: logging.Logger, level: int, obj: Any):
-
     console = Console(width=None)
 
     with console.capture() as capture:
@@ -2141,6 +2190,72 @@ def is_package_installed(package_name):
     return importlib.util.find_spec(package_name) is not None
 
 
+def get_logging_level(level: str | int):
+    """
+    Convert a logging level to its integer representation.
+
+    This function normalizes various logging level inputs (string names,
+    integer values, or custom level strings) into their corresponding
+    integer logging levels.
+
+    Args:
+        level (str | int): The logging level to convert. Can be:
+            - Standard logging level name (e.g., 'DEBUG', 'INFO', 'WARNING')
+            - Integer logging level (e.g., 10, 20, 30)
+            - Custom level string (e.g., 'Level 25')
+
+    Returns:
+        int: The integer representation of the logging level.
+            - Standard levels: DEBUG=10, INFO=20, WARNING=30, ERROR=40, CRITICAL=50
+            - Custom levels: Extracted integer value or dynamically resolved value
+    """
+
+    log_level = logging.getLevelName(level)
+
+    if isinstance(log_level, str):
+        match = re.search(r"\d+", log_level)
+        if match:
+            int_level = level = int(match.group())
+        else:
+            int_level = getattr(logging, log_level)
+    else:
+        int_level = log_level
+
+    return int_level
+
+
+def camel_to_kebab(camel_str: str) -> str:
+    """Convert a string in CamelCase to kebab-case."""
+
+    # Handle sequences of uppercase letters followed by lowercase
+    s1 = re.sub("([A-Z]+)([A-Z][a-z])", r"\1-\2", camel_str)
+
+    # Handle lowercase/digit followed by uppercase
+    s2 = re.sub("([a-z0-9])([A-Z])", r"\1-\2", s1)
+    return s2.lower()
+
+
+def camel_to_snake(camel_str: str) -> str:
+    """Convert a string in CamelCase to snake_case."""
+
+    # Handle sequences of uppercase letters followed by lowercase
+    s1 = re.sub("([A-Z]+)([A-Z][a-z])", r"\1_\2", camel_str)
+
+    # Handle lowercase/digit followed by uppercase
+    s2 = re.sub("([a-z0-9])([A-Z])", r"\1_\2", s1)
+    return s2.lower()
+
+
+def kebab_to_title(kebab_str: str) -> str:
+    """Convert kebab-case to Title Case (each word capitalized)"""
+    return kebab_str.replace("-", " ").title()
+
+
+def snake_to_title(snake_str: str) -> str:
+    """Convert snake_case to Title Case (each word capitalized)"""
+    return snake_str.replace("_", " ").title()
+
+
 ignore_m_warning("egse.system")
 
 if __name__ == "__main__":
